# Generated by Django 3.1.6 on 2021-02-25 03:04

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('edc_action_item', '0028_auto_20210203_0706'),
        ('mocca_ae', '0002_auto_20210129_0556'),
    ]

    operations = [
        migrations.AlterField(
            model_name='aefollowup',
            name='action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='aefollowup',
            name='parent_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='aefollowup',
            name='parent_action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='aefollowup',
            name='related_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='aefollowup',
            name='related_action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='aeinitial',
            name='action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='aeinitial',
            name='parent_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='aeinitial',
            name='parent_action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='aeinitial',
            name='related_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='aeinitial',
            name='related_action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='aesusar',
            name='action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='aesusar',
            name='parent_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='aesusar',
            name='parent_action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='aesusar',
            name='related_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='aesusar',
            name='related_action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='aetmg',
            name='action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='aetmg',
            name='parent_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='aetmg',
            name='parent_action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='aetmg',
            name='related_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='aetmg',
            name='related_action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='deathreport',
            name='action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='deathreport',
            name='parent_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='deathreport',
            name='parent_action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='deathreport',
            name='related_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='deathreport',
            name='related_action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='deathreporttmg',
            name='action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='deathreporttmg',
            name='parent_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='deathreporttmg',
            name='parent_action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='deathreporttmg',
            name='related_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='deathreporttmg',
            name='related_action_item',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to='edc_action_item.actionitem'),
        ),
        migrations.AlterField(
            model_name='historicalaefollowup',
            name='parent_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='historicalaefollowup',
            name='related_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='historicalaeinitial',
            name='parent_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='historicalaeinitial',
            name='related_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='historicalaesusar',
            name='parent_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='historicalaesusar',
            name='related_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='historicalaetmg',
            name='parent_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='historicalaetmg',
            name='related_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='historicaldeathreport',
            name='parent_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='historicaldeathreport',
            name='related_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='historicaldeathreporttmg',
            name='parent_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='historicaldeathreporttmg',
            name='related_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='historicaldeathreporttmgsecond',
            name='parent_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to parent reference model instance.', max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name='historicaldeathreporttmgsecond',
            name='related_action_identifier',
            field=models.CharField(blank=True, help_text='action identifier that links to related reference model instance.', max_length=30, null=True),
        ),
    ]



import os  # noqa

from multisite import SiteID

from .defaults import *  # noqa

print(f"Settings file {__file__}")  # noqa

SITE_ID = SiteID(default=122)
EDC_SITES_UAT_DOMAIN = False
DEBUG = True
ALLOWED_HOSTS = [
    # "amana.tz.mocca.clinicedc.org",  # 220
    # "bunju.tz.mocca.clinicedc.org",  # 250
    # "kisugu.ug.mocca.clinicedc.org",  # 119
    # "kiswa.ug.mocca.clinicedc.org",  # 120
    "localhost",
    "127.0.0.1",
]
# comment: comment out if using runserver and folders read from .env
# ETC_DIR = os.path.join(BASE_DIR, "tests", "etc")  # noqa
# KEY_PATH = os.path.join(ETC_DIR, "crypto_fields")  # noqa
# end comment
AUTO_CREATE_KEYS = False
if os.path.exists(BASE_DIR) and not os.path.exists(KEY_PATH):  # noqa
    os.makedirs(KEY_PATH)  # noqa
    AUTO_CREATE_KEYS = True



from django.db import models
from edc_action_item.models.action_model_mixin import ActionModelMixin
from edc_identifier.model_mixins import (
    NonUniqueSubjectIdentifierFieldMixin,
    TrackingModelMixin,
)
from edc_ltfu.constants import LTFU_ACTION
from edc_ltfu.model_mixins import LtfuModelMixin
from edc_model.models.base_uuid_model import BaseUuidModel
from edc_sites.models import CurrentSiteManager, SiteModelMixin
from edc_visit_schedule.model_mixins import VisitScheduleFieldsModelMixin


class LossToFollowup(
    NonUniqueSubjectIdentifierFieldMixin,
    LtfuModelMixin,
    VisitScheduleFieldsModelMixin,
    SiteModelMixin,
    ActionModelMixin,
    TrackingModelMixin,
    BaseUuidModel,
):

    action_name = LTFU_ACTION

    tracking_identifier_prefix = "LF"

    on_site = CurrentSiteManager()

    class Meta(LtfuModelMixin.Meta, BaseUuidModel.Meta):
        indexes = [
            models.Index(fields=["subject_identifier", "action_identifier", "site", "id"])
        ]



from django.db import models
from django_crypto_fields.fields import EncryptedTextField
from edc_constants.choices import ALIVE_DEAD_UNKNOWN_NA, YES_NO, YES_NO_UNSURE_NA
from edc_constants.constants import NOT_APPLICABLE
from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_sites.models import CurrentSiteManager, SiteModelMixin
from edc_utils import get_utcnow

from ..choices import RESPONDENT_CHOICES
from .mocca_register import MoccaRegister


class Manager(models.Manager):
    """A manager class for Crf models, models that have an FK to
    the visit model.
    """

    use_in_migrations = True

    def get_by_natural_key(self, mocca_register):
        return self.get(mocca_register=mocca_register)


class MoccaRegisterContact(SiteModelMixin, BaseUuidModel):

    mocca_register = models.ForeignKey(MoccaRegister, on_delete=models.PROTECT)

    report_datetime = models.DateTimeField(default=get_utcnow)

    answered = models.CharField(max_length=15, choices=YES_NO, null=True, blank=False)

    respondent = models.CharField(
        max_length=15, choices=RESPONDENT_CHOICES, default=NOT_APPLICABLE
    )

    survival_status = models.CharField(
        max_length=15,
        choices=ALIVE_DEAD_UNKNOWN_NA,
        default=NOT_APPLICABLE,
    )

    death_date = models.DateField(verbose_name="Date of death", null=True, blank=True)

    willing_to_attend = models.CharField(
        max_length=15, choices=YES_NO_UNSURE_NA, default=NOT_APPLICABLE
    )

    icc = models.CharField(
        verbose_name="Does the patient currently receive regular integrated care",
        max_length=25,
        choices=YES_NO_UNSURE_NA,
        null=True,
        blank=False,
        help_text="Either at this facility or elsewhere",
    )

    next_appt_date = models.DateField(verbose_name="Next Appt.", null=True, blank=True)

    call_again = models.CharField(verbose_name="Call again?", max_length=15, choices=YES_NO)

    comment = EncryptedTextField(verbose_name="Note", null=True, blank=True)

    on_site = CurrentSiteManager()
    objects = Manager()
    history = HistoricalRecords()

    def __str__(self):
        return str(self.mocca_register)

    def natural_key(self):
        return (self.mocca_register,)

    natural_key.dependencies = [
        "sites.Site",
        "mocca_screening.MoccaRegister",
    ]

    class Meta:
        verbose_name = "MOCCA Patient Register Contact"
        verbose_name_plural = "MOCCA Patient Register Contacts"
        ordering = ["report_datetime"]



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_subject_admin
from ..forms import ClinicalReviewBaselineForm
from ..models import ClinicalReviewBaseline
from .modeladmin_mixins import CrfModelAdminMixin


@admin.register(ClinicalReviewBaseline, site=mocca_subject_admin)
class ClinicalReviewBaselineAdmin(
    CrfModelAdminMixin, FormLabelModelAdminMixin, SimpleHistoryAdmin
):

    form = ClinicalReviewBaselineForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        ("HIV", {"fields": ("hiv_test", "hiv_test_ago", "hiv_test_date", "hiv_dx")}),
        ("Diabetes", {"fields": ("dm_test", "dm_test_ago", "dm_test_date", "dm_dx")}),
        (
            "Hypertension",
            {"fields": ("htn_test", "htn_test_ago", "htn_test_date", "htn_dx")},
        ),
        (
            "High Cholesterol",
            {"fields": ("chol_test", "chol_test_ago", "chol_test_date", "chol_dx")},
        ),
        ("Complications", {"fields": ("complications",)}),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "complications": admin.VERTICAL,
        "crf_status": admin.VERTICAL,
        "chol_dx": admin.VERTICAL,
        "chol_test": admin.VERTICAL,
        "dm_dx": admin.VERTICAL,
        "dm_test": admin.VERTICAL,
        "hiv_dx": admin.VERTICAL,
        "hiv_test": admin.VERTICAL,
        "htn_dx": admin.VERTICAL,
        "htn_test": admin.VERTICAL,
    }



# Generated by Django 3.0.9 on 2020-12-14 23:41

import edc_model.models.fields.duration
import edc_model.models.validators.date
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_subject", "0002_remove_subjectvisit_health_services"),
    ]

    operations = [
        migrations.AddField(
            model_name="clinicalreviewbaseline",
            name="chol_dx",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="If yes, complete form `High Cholesterol Initial Review`",
                max_length=15,
                verbose_name="Have you ever been diagnosed with High Cholesterol",
            ),
        ),
        migrations.AddField(
            model_name="clinicalreviewbaseline",
            name="chol_test",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="-",
                max_length=15,
                verbose_name="Has the patient ever tested for High Cholesterol?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="clinicalreviewbaseline",
            name="chol_test_ago",
            field=edc_model.models.fields.duration.DurationYMDField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="clinicalreviewbaseline",
            name="chol_test_date",
            field=models.DateField(
                blank=True,
                null=True,
                validators=[edc_model.models.validators.date.date_not_future],
                verbose_name="Date of patient's most recent Cholesterol test?",
            ),
        ),
        migrations.AddField(
            model_name="clinicalreviewbaseline",
            name="chol_test_estimated_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="calculated by the EDC using `chol_test_ago`",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="historicalclinicalreviewbaseline",
            name="chol_dx",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("N/A", "Not applicable")],
                default="N/A",
                help_text="If yes, complete form `High Cholesterol Initial Review`",
                max_length=15,
                verbose_name="Have you ever been diagnosed with High Cholesterol",
            ),
        ),
        migrations.AddField(
            model_name="historicalclinicalreviewbaseline",
            name="chol_test",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                default="-",
                max_length=15,
                verbose_name="Has the patient ever tested for High Cholesterol?",
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="historicalclinicalreviewbaseline",
            name="chol_test_ago",
            field=edc_model.models.fields.duration.DurationYMDField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="historicalclinicalreviewbaseline",
            name="chol_test_date",
            field=models.DateField(
                blank=True,
                null=True,
                validators=[edc_model.models.validators.date.date_not_future],
                verbose_name="Date of patient's most recent Cholesterol test?",
            ),
        ),
        migrations.AddField(
            model_name="historicalclinicalreviewbaseline",
            name="chol_test_estimated_datetime",
            field=models.DateTimeField(
                blank=True,
                help_text="calculated by the EDC using `chol_test_ago`",
                null=True,
            ),
        ),
    ]



from django.contrib import admin
from edc_form_label import FormLabelModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_subject_admin
from ..forms import GlucoseFollowupForm
from ..models import GlucoseFollowup
from .modeladmin_mixins import CrfModelAdminMixin, GlucoseModelAdminMixin


@admin.register(GlucoseFollowup, site=mocca_subject_admin)
class GlucoseFollowupAdmin(
    GlucoseModelAdminMixin,
    CrfModelAdminMixin,
    FormLabelModelAdminMixin,
    SimpleHistoryAdmin,
):
    form = GlucoseFollowupForm



from django.contrib import admin
from edc_ltfu.modeladmin_mixin import LtfuModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ..admin_site import mocca_prn_admin
from ..forms import LossToFollowupForm
from ..models import LossToFollowup


@admin.register(LossToFollowup, site=mocca_prn_admin)
class LossToFollowupAdmin(
    LtfuModelAdminMixin, ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin
):

    form = LossToFollowupForm



from .care_status_form import CareStatusForm
from .daily_closing_log_form import DailyClosingLogForm
from .mocca_register_contact_form import MoccaRegisterContactForm
from .mocca_register_form import MoccaRegisterForm
from .subject_refusal_form import SubjectRefusalForm
from .subject_refusal_screening_form import SubjectRefusalScreeningForm
from .subject_screening_form import SubjectScreeningForm



from edc_visit_schedule import VisitSchedule, site_visit_schedules

from .schedule import schedule

VISIT_SCHEDULE = "visit_schedule"

visit_schedule = VisitSchedule(
    name=VISIT_SCHEDULE,
    verbose_name="MOCCA",
    offstudy_model="edc_offstudy.subjectoffstudy",
    death_report_model="mocca_ae.deathreport",
    locator_model="edc_locator.subjectlocator",
    previous_visit_schedule=None,
)

visit_schedule.add_schedule(schedule)

site_visit_schedules.register(visit_schedule)



# Generated by Django 3.0.9 on 2020-12-04 00:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0007_auto_20201204_0232"),
    ]

    operations = [
        migrations.RenameField(
            model_name="historicalmoccaregistercontact",
            old_name="contacted",
            new_name="answered",
        ),
        migrations.RenameField(
            model_name="moccaregistercontact",
            old_name="contacted",
            new_name="answered",
        ),
        migrations.AddField(
            model_name="historicalmoccaregistercontact",
            name="respondent",
            field=models.CharField(
                choices=[
                    ("patient", "Patient"),
                    ("family", "Family"),
                    ("friend", "friend"),
                    ("other", "other"),
                ],
                default="No",
                max_length=15,
            ),
        ),
        migrations.AddField(
            model_name="moccaregistercontact",
            name="respondent",
            field=models.CharField(
                choices=[
                    ("patient", "Patient"),
                    ("family", "Family"),
                    ("friend", "friend"),
                    ("other", "other"),
                ],
                default="No",
                max_length=15,
            ),
        ),
        migrations.AlterField(
            model_name="historicalmoccaregistercontact",
            name="call_again",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Try again?",
            ),
        ),
        migrations.AlterField(
            model_name="historicalmoccaregistercontact",
            name="survival_status",
            field=models.CharField(
                choices=[
                    ("alive", "Alive"),
                    ("dead", "Deceased"),
                    ("unknown", "Unknown"),
                ],
                default="unknown",
                max_length=15,
            ),
        ),
        migrations.AlterField(
            model_name="historicalmoccaregistercontact",
            name="willing_to_attend",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("unknown", "Unknown")],
                default="unknown",
                max_length=15,
            ),
        ),
        migrations.AlterField(
            model_name="moccaregistercontact",
            name="call_again",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=15,
                verbose_name="Try again?",
            ),
        ),
        migrations.AlterField(
            model_name="moccaregistercontact",
            name="survival_status",
            field=models.CharField(
                choices=[
                    ("alive", "Alive"),
                    ("dead", "Deceased"),
                    ("unknown", "Unknown"),
                ],
                default="unknown",
                max_length=15,
            ),
        ),
        migrations.AlterField(
            model_name="moccaregistercontact",
            name="willing_to_attend",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No"), ("unknown", "Unknown")],
                default="unknown",
                max_length=15,
            ),
        ),
    ]



from django import forms
from edc_constants.constants import YES
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx_review.utils import raise_if_clinical_review_does_not_exist
from edc_form_validators.form_validator import FormValidator

from ..models import ComplicationsFollowup


class ComplicationsFollowupFormValidators(CrfFormValidatorMixin, FormValidator):
    def clean(self):
        raise_if_clinical_review_does_not_exist(self.cleaned_data.get("subject_visit"))
        self.required_if(YES, field="stroke", field_required="stroke_date")
        self.required_if(YES, field="heart_attack", field_required="heart_attack_date")
        self.required_if(YES, field="renal_disease", field_required="renal_disease_date")
        self.required_if(YES, field="vision", field_required="vision_date")
        self.required_if(YES, field="numbness", field_required="numbness_date")
        self.required_if(YES, field="foot_ulcers", field_required="foot_ulcers_date")
        self.required_if(YES, field="complications", field_required="complications_other")


class ComplicationsFollowupForm(CrfModelFormMixin, forms.ModelForm):

    form_validator_cls = ComplicationsFollowupFormValidators

    class Meta:
        model = ComplicationsFollowup
        fields = "__all__"



from django.urls.conf import path
from django.views.generic import RedirectView

app_name = "mocca_screening"

urlpatterns = [
    path("", RedirectView.as_view(url=f"/{app_name}/admin/"), name="home_url"),
]



from django.contrib import admin
from django.utils.html import format_html
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin
from edc_phq9.fieldsets import get_phq9_fieldsets
from edc_phq9.model_admin_mixins import get_phq9_radio_fields

from ..admin_site import mocca_subject_admin
from ..forms import PatientHealthForm
from ..models import PatientHealth
from .modeladmin_mixins import CrfModelAdminMixin

radio_fields = get_phq9_radio_fields()
radio_fields.update({"crf_status": admin.VERTICAL, "ph9_performed": admin.VERTICAL})


@admin.register(PatientHealth, site=mocca_subject_admin)
class PatientHealthAdmin(CrfModelAdminMixin, FormLabelModelAdminMixin, SimpleHistoryAdmin):
    form = PatientHealthForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_visit",
                    "report_datetime",
                    "ph9_performed",
                    "ph9_not_performed_reason",
                )
            },
        ),
        get_phq9_fieldsets(),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = radio_fields



import re

from django.db.models import Q
from edc_dashboard.view_mixins import (
    EdcViewMixin,
    ListboardFilterViewMixin,
    SearchFormViewMixin,
)
from edc_dashboard.views import ListboardView
from edc_navbar import NavbarViewMixin
from edc_subject_model_wrappers import SubjectConsentModelWrapper


class ListboardView(
    EdcViewMixin,
    NavbarViewMixin,
    ListboardFilterViewMixin,
    SearchFormViewMixin,
    ListboardView,
):

    listboard_template = "subject_listboard_template"
    listboard_url = "subject_listboard_url"
    listboard_panel_style = "success"
    listboard_fa_icon = "far fa-user-circle"
    listboard_model = "mocca_consent.subjectconsent"
    listboard_view_permission_codename = "edc_dashboard.view_subject_listboard"

    model_wrapper_cls = SubjectConsentModelWrapper
    navbar_selected_item = "consented_subject"
    search_form_url = "subject_listboard_url"

    def get_queryset_filter_options(self, request, *args, **kwargs):
        options = super().get_queryset_filter_options(request, *args, **kwargs)
        if kwargs.get("subject_identifier"):
            options.update({"subject_identifier": kwargs.get("subject_identifier")})
        return options

    def extra_search_options(self, search_term):
        q_objects = []
        if re.match("^[A-Za-z\-]+$", search_term):
            q_objects.append(Q(initials__exact=search_term.upper()))
            q_objects.append(Q(first_name__exact=search_term.upper()))
            q_objects.append(
                Q(screening_identifier__icontains=search_term.replace("-", "").upper())
            )
            q_objects.append(Q(subject_identifier__icontains=search_term))
        if re.match("^[0-9]+$", search_term):
            q_objects.append(Q(identity__exact=search_term))
        return q_objects



from django import forms
from django.conf import settings
from edc_consent.form_validators import SubjectConsentFormValidatorMixin
from edc_consent.modelform_mixins import ConsentModelFormMixin
from edc_form_validators import FormValidator, FormValidatorMixin
from edc_sites.forms import SiteModelFormMixin
from edc_utils import convert_php_dateformat

from ..models import SubjectConsent


class SubjectConsentFormValidator(SubjectConsentFormValidatorMixin, FormValidator):
    subject_screening_model = "mocca_screening.subjectscreening"

    def validate_consent_datetime(self):
        """Validate consent datetime with the report_datetime
        instead of eligibility datetime.

        report_datetime must come first.

        Watchout for timezone, cleaned_data has local TZ.
        """

        if (
            self.consent_datetime - self.subject_screening.report_datetime
        ).total_seconds() < 0:
            local_dt = self.subject_screening.report_datetime.astimezone(self.tz)
            formatted = local_dt.strftime(
                convert_php_dateformat(settings.SHORT_DATETIME_FORMAT)
            )
            raise forms.ValidationError(
                {
                    "consent_datetime": (
                        f"Cannot be before the date and time screening "
                        f"information was reported. Report datetime was "
                        f"{formatted}."
                    )
                },
            )


class SubjectConsentForm(
    SiteModelFormMixin, FormValidatorMixin, ConsentModelFormMixin, forms.ModelForm
):
    form_validator_cls = SubjectConsentFormValidator

    screening_identifier = forms.CharField(
        label="Screening identifier",
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    def clean_gender_of_consent(self):
        """Skip. Validated on form"""
        return None

    def clean_guardian_and_dob(self):
        """Skip. Minors not included in this trial"""
        return None

    class Meta:
        model = SubjectConsent
        fields = [
            "assessment_score",
            "confirm_identity",
            "consent_copy",
            "consent_datetime",
            "consent_reviewed",
            "consent_signature",
            "dob",
            "first_name",
            "gender",
            "identity",
            "identity_type",
            "initials",
            "is_dob_estimated",
            "is_incarcerated",
            "is_literate",
            "language",
            "last_name",
            "screening_identifier",
            "study_questions",
            "witness_name",
            "user_created",  # added for tests
        ]
        help_texts = {
            "identity": (
                "Use Country ID Number, Passport number, driver's license "
                "number, Mobile, etc"
            ),
            "witness_name": (
                "Required only if participant is illiterate. "
                "Format is 'LASTNAME, FIRSTNAME'. "
                "All uppercase separated by a comma."
            ),
        }



from dateutil.relativedelta import relativedelta
from edc_ltfu.utils import get_ltfu_model_name
from edc_visit_schedule import Schedule
from edc_visit_schedule import Visit as BaseVisit
from edc_visit_schedule.constants import DAY1

from ..constants import SCHEDULE
from .crfs import crfs, crfs_d1, crfs_missed
from .crfs import crfs_prn as default_crfs_prn
from .crfs import crfs_unscheduled as default_crfs_unscheduled
from .requisitions import requisitions_all, requisitions_d1, requisitions_prn

default_requisitions = None


class Visit(BaseVisit):
    def __init__(
        self,
        crfs_unscheduled=None,
        requisitions_unscheduled=None,
        crfs_prn=None,
        requisitions_prn=None,
        allow_unscheduled=None,
        **kwargs
    ):
        super().__init__(
            allow_unscheduled=True if allow_unscheduled is None else allow_unscheduled,
            crfs_unscheduled=crfs_unscheduled or default_crfs_unscheduled,
            requisitions_unscheduled=requisitions_unscheduled or default_requisitions,
            crfs_prn=crfs_prn or default_crfs_prn,
            requisitions_prn=requisitions_prn,  # or default_requisitions_prn,
            crfs_missed=crfs_missed,
            **kwargs,
        )


# schedule for new participants
schedule = Schedule(
    name=SCHEDULE,
    verbose_name="Day 1 to Month 12",
    onschedule_model="mocca_prn.onschedule",
    offschedule_model="mocca_prn.offschedule",
    consent_model="mocca_consent.subjectconsent",
    appointment_model="edc_appointment.appointment",
    loss_to_followup_model=get_ltfu_model_name(),
)

visit00 = Visit(
    code=DAY1,
    title="Day 1",
    timepoint=0,
    rbase=relativedelta(days=0),
    rlower=relativedelta(days=0),
    rupper=relativedelta(days=0),
    requisitions=requisitions_d1,
    requisitions_prn=requisitions_prn,
    crfs=crfs_d1,
    facility_name="5-day-clinic",
)

schedule.add_visit(visit=visit00)

for n in range(1, 13):
    visit = Visit(
        code=str(1000 + (10 * n)),
        title=f"Month {n}",
        timepoint=n,
        rbase=relativedelta(months=n),
        rlower=relativedelta(months=0),
        rupper=relativedelta(months=n + 1),
        requisitions=requisitions_all,
        requisitions_prn=requisitions_prn,
        crfs=crfs,
        facility_name="5-day-clinic",
    )
    schedule.add_visit(visit=visit)



from django.apps import apps as django_apps
from edc_constants.constants import CLOSED, OTHER
from edc_form_validators import FormValidator

from .study_day_form_validator_mixin import StudyDayFormValidatorMixin


class DeathReportFormValidator(StudyDayFormValidatorMixin, FormValidator):
    @property
    def cause_of_death_model_cls(self):
        return django_apps.get_model("edc_adverse_event.causeofdeath")

    def clean(self):

        self.validate_study_day_with_datetime(
            study_day=self.cleaned_data.get("study_day"),
            compare_date=self.cleaned_data.get("death_datetime"),
            study_day_field="study_day",
        )

        other = self.cause_of_death_model_cls.objects.get(name=OTHER)
        self.validate_other_specify(
            field="cause_of_death",
            other_specify_field="cause_of_death_other",
            other_stored_value=other.name,
        )

        self.required_if(
            CLOSED, field="report_status", field_required="report_closed_datetime"
        )



from django.core.exceptions import ValidationError
from django.db.models.signals import post_delete, post_save
from django.dispatch import receiver
from edc_randomization.site_randomizers import site_randomizers
from edc_screening.utils import get_subject_screening_model_cls
from edc_visit_schedule.site_visit_schedules import site_visit_schedules
from edc_visit_tracking.utils import get_subject_visit_model

from .subject_consent import SubjectConsent


def get_onschedule_model_name(instance):
    onschedule_model_name = "mocca_prn.onschedule"
    return onschedule_model_name


@receiver(
    post_save,
    weak=False,
    sender=SubjectConsent,
    dispatch_uid="subject_consent_on_post_save",
)
def subject_consent_on_post_save(sender, instance, raw, created, **kwargs):
    """Creates an onschedule instance for this consented subject, if
    it does not exist.
    """
    if not raw:
        if not created:
            _, schedule = site_visit_schedules.get_by_onschedule_model(
                get_onschedule_model_name(instance)
            )
            schedule.refresh_schedule(subject_identifier=instance.subject_identifier)
        else:
            subject_screening = get_subject_screening_model_cls().objects.get(
                screening_identifier=instance.screening_identifier
            )
            subject_screening.subject_identifier = instance.subject_identifier
            subject_screening.consented = True
            subject_screening.save_base(update_fields=["subject_identifier", "consented"])

            # randomize
            # TODO: should get randomizer name "default" from model or Consent object
            site_randomizers.randomize(
                "default",
                subject_identifier=instance.subject_identifier,
                report_datetime=instance.consent_datetime,
                site=instance.site,
                user=instance.user_created,
            )

            # put subject on primary schedule
            _, schedule = site_visit_schedules.get_by_onschedule_model(
                get_onschedule_model_name(instance)
            )
            schedule.put_on_schedule(
                subject_identifier=instance.subject_identifier,
                onschedule_datetime=instance.consent_datetime,
            )


@receiver(
    post_delete,
    weak=False,
    sender=SubjectConsent,
    dispatch_uid="subject_consent_on_post_delete",
)
def subject_consent_on_post_delete(sender, instance, using, **kwargs):
    """Updates/Resets subject screening."""
    # don't allow if subject visits exist. This should be caught
    # in the ModelAdmin delete view
    subject_visit_model_cls = get_subject_visit_model()
    if subject_visit_model_cls.objects.filter(
        subject_identifier=instance.subject_identifier
    ).exists():
        raise ValidationError("Unable to delete consent. Visit data exists.")

    _, schedule = site_visit_schedules.get_by_onschedule_model(
        get_onschedule_model_name(instance)
    )
    schedule.take_off_schedule(
        subject_identifier=instance.subject_identifier,
        offschedule_datetime=instance.consent_datetime,
    )

    # update subject screening
    subject_screening = get_subject_screening_model_cls().objects.get(
        screening_identifier=instance.screening_identifier
    )
    subject_screening.consented = False
    subject_screening.subject_identifier = subject_screening.subject_screening_as_pk
    subject_screening.save()



from django.core.validators import MaxValueValidator, MinValueValidator
from django.db import models
from edc_lab.choices import VL_QUANTIFIER_NA
from edc_lab.constants import EQ
from edc_model import models as edc_models
from edc_reportable import COPIES_PER_MILLILITER

from ..model_mixins import CrfModelMixin


class ViralLoadResult(CrfModelMixin, edc_models.BaseUuidModel):

    drawn_date = models.DateField(
        verbose_name="Specimen collection date",
        validators=[edc_models.date_not_future],
    )

    result = models.IntegerField(
        verbose_name="VL Result",
        validators=[MinValueValidator(20), MaxValueValidator(9999999)],
        help_text=f"in {COPIES_PER_MILLILITER}",
    )

    quantifier = models.CharField(max_length=10, choices=VL_QUANTIFIER_NA, default=EQ)

    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Viral Load Result"
        verbose_name_plural = "Viral Load Results"



from django.core.exceptions import ImproperlyConfigured
from edc_action_item import Action
from edc_action_item.utils import register_actions
from edc_adverse_event.constants import AE_INITIAL_ACTION
from edc_blood_results import BLOOD_RESULTS_LIPID_ACTION
from edc_constants.constants import HIGH_PRIORITY, YES
from edc_ltfu.constants import LTFU_ACTION
from edc_visit_schedule.utils import is_baseline
from edc_visit_tracking.action_items import VisitMissedAction

from mocca_visit_schedule.constants import SCHEDULE


class SubjectVisitMissedAction(VisitMissedAction):
    reference_model = "mocca_subject.subjectvisitmissed"
    admin_site_name = "mocca_subject_admin"
    loss_to_followup_action_name = None

    def get_loss_to_followup_action_name(self):
        schedule = self.reference_obj.visit.appointment.schedule
        if schedule.name == SCHEDULE:
            return LTFU_ACTION
        raise ImproperlyConfigured(
            "Unable to determine action name. Schedule name not known. "
            f"Got {schedule.name}."
        )


class BaseBloodResultsAction(Action):
    name = None
    display_name = None
    reference_model = None

    priority = HIGH_PRIORITY
    show_on_dashboard = True
    create_by_user = False

    def reopen_action_item_on_change(self):
        return False

    def get_next_actions(self):
        next_actions = []
        if (
            self.reference_obj.results_abnormal == YES
            and self.reference_obj.results_reportable == YES
            and not is_baseline(self.reference_obj_has_changed)
        ):
            # AE for reportable result, though not on DAY1.0
            next_actions = [AE_INITIAL_ACTION]
        return next_actions


class BloodResultsLipidAction(BaseBloodResultsAction):
    name = BLOOD_RESULTS_LIPID_ACTION
    display_name = "Reportable result: LIPIDS"
    reference_model = "mocca_subject.bloodresultslipid"


register_actions(BloodResultsLipidAction, SubjectVisitMissedAction)



from django.contrib import admin
from edc_consent.modeladmin_mixins import ModelAdminConsentMixin
from edc_model_admin import SimpleHistoryAdmin, audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin

from ..admin_site import mocca_consent_admin
from ..forms import SubjectConsentForm
from ..models import SubjectConsent


@admin.register(SubjectConsent, site=mocca_consent_admin)
class SubjectConsentAdmin(
    ModelAdminConsentMixin, ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin
):
    form = SubjectConsentForm

    fieldsets = (
        (
            None,
            {
                "fields": (
                    "subject_identifier",
                    "screening_identifier",
                    "first_name",
                    "last_name",
                    "initials",
                    "gender",
                    "language",
                    "is_literate",
                    "witness_name",
                    "consent_datetime",
                    "dob",
                    "is_dob_estimated",
                    "identity",
                    "identity_type",
                    "confirm_identity",
                    "is_incarcerated",
                )
            },
        ),
        (
            "Review Questions",
            {
                "fields": (
                    "consent_reviewed",
                    "study_questions",
                    "assessment_score",
                    "consent_signature",
                    "consent_copy",
                ),
                "description": "The following questions are directed to the interviewer.",
            },
        ),
        audit_fieldset_tuple,
    )

    search_fields = ("subject_identifier", "screening_identifier", "identity")

    radio_fields = {
        "assessment_score": admin.VERTICAL,
        "consent_copy": admin.VERTICAL,
        "consent_reviewed": admin.VERTICAL,
        "consent_signature": admin.VERTICAL,
        "gender": admin.VERTICAL,
        "is_dob_estimated": admin.VERTICAL,
        "identity_type": admin.VERTICAL,
        "is_incarcerated": admin.VERTICAL,
        "is_literate": admin.VERTICAL,
        "language": admin.VERTICAL,
        "study_questions": admin.VERTICAL,
    }



from edc_constants.constants import NOT_APPLICABLE, OTHER
from edc_protocol_violation.constants import DEVIATION, VIOLATION

LOSS_CHOICES = (
    ("unknown_address", "Changed to an unknown address"),
    ("never_returned", "Did not return despite reminders"),
    ("bad_contact_details", "Inaccurate contact details"),
    (OTHER, "Other"),
)

PROTOCOL_VIOLATION = (
    ("failure_to_obtain_informed_consent", "Failure to obtain informed " "consent"),
    ("enrollment_of_ineligible_patient", "Enrollment of ineligible patient"),
    (
        "screening_procedure not done",
        "Screening procedure required by " "protocol not done",
    ),
    (
        "screening_or_on-study_procedure",
        "Screening or on-study procedure/lab " "work required not done",
    ),
    (
        "incorrect_research_treatment",
        "Incorrect research treatment given to " "patient",
    ),
    (
        "procedure_not_completed",
        "On-study procedure required by protocol not " "completed",
    ),
    ("visit_non-compliance", "Visit non-compliance"),
    ("medication_stopped_early", "Medication stopped early"),
    ("medication_noncompliance", "Medication_noncompliance"),
    (
        "national_regulations_not_met",
        "Standard WPD, ICH-GCP, local/national " "regulations not met",
    ),
    (OTHER, "Other"),
    (NOT_APPLICABLE, "Not applicable"),
)

ACTION_REQUIRED = (
    ("remain_on_study", "Participant to remain on trial"),
    ("to_be_withdrawn", "Participant to be withdrawn from trial"),
    (
        "remain_on_study_modified",
        "Patient remains on study but data analysis will be modified",
    ),
)

DEVIATION_VIOLATION = (
    (VIOLATION, "Protocol violation"),
    (DEVIATION, "Protocol deviation"),
)


REASON_STUDY_TERMINATED = ()



from django.conf import settings
from edc_lab import LabProfile
from edc_lab_panel.panels import (
    blood_glucose_panel,
    blood_glucose_poc_panel,
    hba1c_panel,
    hba1c_poc_panel,
)

from .panels import lipids_panel

subject_lab_profile = LabProfile(
    name="subject_lab_profile", requisition_model=settings.SUBJECT_REQUISITION_MODEL
)

subject_lab_profile.add_panel(blood_glucose_panel)
subject_lab_profile.add_panel(blood_glucose_poc_panel)
subject_lab_profile.add_panel(hba1c_panel)
subject_lab_profile.add_panel(hba1c_poc_panel)
subject_lab_profile.add_panel(lipids_panel)



from django.test import TestCase, tag

from ..visit_schedules.schedule import schedule
from ..visit_schedules.visit_schedule import visit_schedule


class TestVisitSchedule(TestCase):
    def test_visit_schedule_models(self):
        self.assertEqual(visit_schedule.death_report_model, "mocca_ae.deathreport")
        self.assertEqual(visit_schedule.offstudy_model, "edc_offstudy.subjectoffstudy")
        self.assertEqual(visit_schedule.locator_model, "edc_locator.subjectlocator")

    def test_schedule_models(self):
        self.assertEqual(schedule.onschedule_model, "mocca_prn.onschedulehiv")
        self.assertEqual(schedule.offschedule_model, "mocca_prn.endofstudy")
        self.assertEqual(schedule.consent_model, "mocca_consent.subjectconsent")
        self.assertEqual(schedule.appointment_model, "edc_appointment.appointment")

    def test_crfs(self):
        prn = [
            "mocca_subject.bloodresultsfbc",
            "mocca_subject.bloodresultsglu",
            "mocca_subject.bloodresultshba1c",
            "mocca_subject.bloodresultslft",
            "mocca_subject.bloodresultsrft",
            "mocca_subject.urinedipsticktest",
        ]
        expected = {
            "1000": [
                "mocca_subject.physicalexam",
                "mocca_subject.patienthistory",
                "mocca_subject.bloodresultsfbc",
                "mocca_subject.bloodresultslft",
                "mocca_subject.bloodresultsrft",
                "mocca_subject.urinedipsticktest",
            ],
            "1005": [
                "mocca_subject.followupvitals",
                "mocca_subject.followup",
                "mocca_subject.clinicalreviewbaseline",
                "mocca_subject.medicationadherence",
            ],
            "1010": [
                "mocca_subject.followupvitals",
                "mocca_subject.followup",
                "mocca_subject.medicationadherence",
            ],
            "1030": [
                "mocca_subject.bloodresultslft",
                "mocca_subject.bloodresultsrft",
                "mocca_subject.followupvitals",
                "mocca_subject.followup",
                "mocca_subject.medicationadherence",
            ],
            "1060": [
                "mocca_subject.bloodresultshba1c",
                "mocca_subject.bloodresultslft",
                "mocca_subject.bloodresultsrft",
                "mocca_subject.followupvitals",
                "mocca_subject.followup",
                "mocca_subject.medicationadherence",
            ],
            "1090": [
                "mocca_subject.bloodresultslft",
                "mocca_subject.bloodresultsrft",
                "mocca_subject.followupvitals",
                "mocca_subject.followup",
                "mocca_subject.medicationadherence",
            ],
            "1120": [
                "mocca_subject.bloodresultsfbc",
                "mocca_subject.bloodresultsglu",
                "mocca_subject.bloodresultshba1c",
                "mocca_subject.bloodresultslft",
                "mocca_subject.bloodresultsrft",
                "mocca_subject.followupvitals",
                "mocca_subject.followup",
                "mocca_subject.medicationadherence",
                "mocca_subject.urinedipsticktest",
            ],
        }
        for visit_code, visit in schedule.visits.items():
            actual = [crf.model for crf in visit.crfs]
            actual.sort()
            expected.get(visit_code).sort()
            self.assertEqual(
                expected.get(visit_code), actual, msg=f"see CRFs for visit {visit_code}"
            )

            actual = [crf.model for crf in visit.crfs_prn]
            actual.sort()
            self.assertEqual(prn, actual, msg=f"see PRN CRFs for visit {visit_code}")



from django.core.management import color_style
from edc_sites.single_site import SingleSite as BaseSingleSite

style = color_style()

fqdn = "mocca.clinicedc.org"


class SingleSite(BaseSingleSite):
    def __init__(self, *args, orig_site_id=None, **kwargs):
        self.orig_site_id = orig_site_id
        super().__init__(*args, **kwargs)


# site_id, name, **kwargs
all_sites = {
    "uganda": (
        SingleSite(
            119,
            "kisugu",
            title="Kisugu Hospital",
            orig_site_id=6,
            country_code="ug",
            country="uganda",
            domain=f"kisugu.ug.{fqdn}",
        ),
        SingleSite(
            120,
            "kiswa",
            title="Kiswa Hospital",
            orig_site_id=7,
            country_code="ug",
            country="uganda",
            domain=f"kiswa.ug.{fqdn}",
        ),
        SingleSite(
            121,
            "mulago",
            title="Mulago Hospital",
            country_code="ug",
            orig_site_id=8,
            country="uganda",
            domain=f"mulago.ug.{fqdn}",
        ),
        SingleSite(
            122,
            "ndejje",
            title="Ndejje Hospital",
            orig_site_id=9,
            country_code="ug",
            country="uganda",
            domain=f"ndejje.ug.{fqdn}",
        ),
        SingleSite(
            123,
            "wakiso",
            title="Wakiso Hospital",
            orig_site_id=10,
            country_code="ug",
            country="uganda",
            domain=f"wakiso.ug.{fqdn}",
        ),
    ),
    "tanzania": (
        SingleSite(
            220,
            "amana",
            title="Amana Hospital",
            orig_site_id=1,
            country="tanzania",
            country_code="tz",
            domain=f"amana.tz.{fqdn}",
        ),
        SingleSite(
            250,
            "bunju",
            title="Bunju Hospital",
            orig_site_id=2,
            country="tanzania",
            country_code="tz",
            domain=f"bunju.tz.{fqdn}",
        ),
        SingleSite(
            221,
            "hindu_mandal",
            title="Hindu Mandal Hospital",
            orig_site_id=3,
            country="tanzania",
            country_code="tz",
            domain=f"hindu-mandal.tz.{fqdn}",
        ),
        SingleSite(
            222,
            "mkuranga",
            title="Mkuranga Hospital",
            orig_site_id=4,
            country="tanzania",
            country_code="tz",
            domain=f"mkuranga.tz.{fqdn}",
        ),
        SingleSite(
            240,
            "mwananyamala",
            title="Mwananyamala Hospital",
            orig_site_id=5,
            country="tanzania",
            country_code="tz",
            domain=f"mwananyamala.tz.{fqdn}",
        ),
    ),
}



# Generated by Django 3.0.9 on 2020-12-05 01:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("mocca_screening", "0016_auto_20201204_2347"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalmoccaregistercontact",
            name="answered",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")], max_length=15, null=True
            ),
        ),
        migrations.AlterField(
            model_name="moccaregistercontact",
            name="answered",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")], max_length=15, null=True
            ),
        ),
    ]



from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_lab.form_validators import RequisitionFormValidator
from edc_lab.forms import RequisitionFormMixin
from edc_metadata.constants import NOT_REQUIRED

from ..models import SubjectRequisition


class SubjectRequisitionForm(RequisitionFormMixin, CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = RequisitionFormValidator

    requisition_identifier = forms.CharField(
        label="Requisition identifier",
        widget=forms.TextInput(attrs={"readonly": "readonly"}),
    )

    def clean(self):
        cleaned_data = super().clean()
        if cleaned_data.get("reason_not_drawn") == NOT_REQUIRED:
            raise forms.ValidationError(
                {"reason_not_drawn": "Invalid choice. Not expected for this panel"}
            )
        return cleaned_data

    @property
    def report_datetime(self):
        return self.cleaned_data.get("requisition_datetime")

    class Meta:
        model = SubjectRequisition
        fields = "__all__"



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_crf.admin import crf_status_fieldset_tuple
from edc_form_label.form_label_modeladmin_mixin import FormLabelModelAdminMixin
from edc_model_admin import SimpleHistoryAdmin

from ..admin_site import mocca_subject_admin
from ..forms import MedicationsForm
from ..models import Medications
from .modeladmin_mixins import CrfModelAdminMixin


@admin.register(Medications, site=mocca_subject_admin)
class MedicationsAdmin(CrfModelAdminMixin, FormLabelModelAdminMixin, SimpleHistoryAdmin):
    form = MedicationsForm

    fieldsets = (
        (None, {"fields": ("subject_visit", "report_datetime")}),
        (
            "Prescriptions",
            {"fields": ("refill_htn", "refill_dm", "refill_chol", "refill_hiv")},
        ),
        crf_status_fieldset_tuple,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "refill_chol": admin.VERTICAL,
        "refill_htn": admin.VERTICAL,
        "refill_dm": admin.VERTICAL,
        "refill_hiv": admin.VERTICAL,
        "crf_status": admin.VERTICAL,
    }



from django import forms
from edc_constants.constants import NO, YES
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx_review.utils import (
    art_initiation_date,
    raise_if_clinical_review_does_not_exist,
)
from edc_form_validators.form_validator import FormValidator

from ..models import HivReview


class HivReviewFormValidator(CrfFormValidatorMixin, FormValidator):
    def clean(self):
        raise_if_clinical_review_does_not_exist(self.cleaned_data.get("subject_visit"))
        self.validate_care_delivery()
        art_init_date = art_initiation_date(
            subject_identifier=self.subject_identifier,
            report_datetime=self.report_datetime,
        )
        self.applicable_if_true(
            not art_init_date,
            field_applicable="arv_initiated",
            applicable_msg="Subject was NOT previously reported as on ART.",
            not_applicable_msg="Subject was previously reported as on ART.",
        )
        self.required_if(
            YES, field="arv_initiated", field_required="arv_initiation_actual_date"
        )

    def validate_care_delivery(self) -> None:
        self.required_if(NO, field="care_delivery", field_required="care_delivery_other")


class HivReviewForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = HivReviewFormValidator

    class Meta:
        model = HivReview
        fields = "__all__"



from django import forms
from edc_constants.constants import ALIVE, DEAD, UNKNOWN, YES
from edc_form_validators import FormValidator, FormValidatorMixin

from ..models import MoccaRegisterContact


class MoccaRegisterContactFormValidator(FormValidator):
    def clean(self):
        self.applicable_if(YES, field="answered", field_applicable="respondent")
        self.applicable_if(YES, field="answered", field_applicable="survival_status")
        self.not_required_if(
            UNKNOWN,
            ALIVE,
            field="survival_status",
            field_required="death_date",
            inverse=False,
        )
        self.applicable_if(
            ALIVE, field="survival_status", field_applicable="willing_to_attend"
        )
        self.applicable_if(ALIVE, field="survival_status", field_applicable="icc")
        self.required_if(YES, field="willing_to_attend", field_required="next_appt_date")
        if (
            self.cleaned_data.get("survival_status") == DEAD
            and self.cleaned_data.get("call_again") == YES
        ):
            raise forms.ValidationError({"call_again": "Invalid. Subject is deceased"})


class MoccaRegisterContactForm(FormValidatorMixin, forms.ModelForm):

    form_validator_cls = MoccaRegisterContactFormValidator

    class Meta:
        model = MoccaRegisterContact
        fields = [
            "answered",
            "respondent",
            "survival_status",
            "death_date",
            "willing_to_attend",
            "icc",
            "next_appt_date",
            "call_again",
            "report_datetime",
        ]
        labels = {"report_datetime": "Date", "answered": "Did someone answer?"}



from django.db import models
from edc_constants.choices import YES_NO
from edc_constants.constants import YES
from edc_model.models import BaseUuidModel, HistoricalRecords
from edc_sites.models import CurrentSiteManager, SiteModelMixin
from edc_utils import get_utcnow

from mocca_screening.models.model_mixins import CareModelMixin


class Manager(models.Manager):
    """A manager class for Crf models, models that have an FK to
    the visit model.
    """

    use_in_migrations = True

    def get_by_natural_key(self, mocca_register):
        return self.get(mocca_register=mocca_register)


class CareStatus(SiteModelMixin, CareModelMixin, BaseUuidModel):

    willing_to_answer = models.CharField(
        verbose_name="Is the patient willing to provide information about their care status?",
        max_length=15,
        choices=YES_NO,
        default=YES,
    )

    report_datetime = models.DateTimeField(
        verbose_name="Report Date and Time",
        default=get_utcnow,
        help_text="Date and time of report.",
    )

    mocca_register = models.OneToOneField(
        "mocca_screening.moccaregister",
        on_delete=models.PROTECT,
        null=True,
        verbose_name="MOCCA (original) register details",
    )

    on_site = CurrentSiteManager()
    objects = Manager()
    history = HistoricalRecords()

    def natural_key(self):
        return tuple(self.mocca_register)

    natural_key.dependencies = [
        "sites.Site",
        "mocca_screen ing.MoccaRegister",
    ]

    class Meta(BaseUuidModel.Meta):
        verbose_name = "Care Status"
        verbose_name_plural = "Care Status"



from django import forms
from edc_appointment.constants import NEW_APPT
from edc_crf.forms import CrfFormValidatorMixin
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx_review.utils import raise_if_clinical_review_does_not_exist
from edc_form_validators.form_validator import FormValidator

from mocca_consent.models import SubjectConsent

from ..models import NextAppointment, SubjectVisit


class NextAppointmentValidator(CrfFormValidatorMixin, FormValidator):
    def clean(self):
        raise_if_clinical_review_does_not_exist(self.cleaned_data.get("subject_visit"))
        self.date_not_before(
            "report_datetime",
            "appt_date",
            convert_to_date=True,
        )
        appointment = self.cleaned_data.get("subject_visit").appointment
        appt_date = self.cleaned_data.get("appt_date")
        # TODO: confirm next appointment date logic for routine appointment
        if appointment.next:
            msg = None
            if (
                appointment.next.appt_status != NEW_APPT
                and appt_date != appointment.next.appt_datetime.date()
            ):
                msg = f"Invalid. Expected {appointment.next.appt_datetime.date()}"
            elif appointment.next.next:
                if appt_date and appointment.next.next.appt_datetime.date():
                    if appt_date > appointment.next.next.appt_datetime.date():
                        msg = (
                            "Invalid. Cannot be after "
                            f"{appointment.next.next.appt_datetime.date()} "
                        )
            if msg:
                raise forms.ValidationError({"appt_date": f"{msg}"})

    @property
    def clinic_type(self):
        return SubjectConsent.objects.get(
            subject_identifier=self.subject_identifier
        ).clinic_type


class NextAppointmentForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = NextAppointmentValidator

    def __init__(self, *args, **kwargs):
        try:
            appt_date = SubjectVisit.objects.get(
                id=kwargs.get("initial").get("subject_visit")
            ).appointment.next.appt_datetime
        except AttributeError:
            pass
        else:
            kwargs["initial"].update(appt_date=appt_date)
        super().__init__(*args, **kwargs)

    class Meta:
        model = NextAppointment
        fields = "__all__"
